' add a COM reference for:
' Microsoft ADO Ext. for DDL and Security (ADOX)
Imports System.IO
Imports System.Data.OleDb

'
' A simple class for dealing with Microsoft Access databases
'
Public Class AccessDatabase
    Private da As New OleDbDataAdapter
    Private con As New OleDbConnection

    '
    ' create a new empty Microsoft Access MDB file
    '
    Public Sub CreateDatabase(ByVal mdb_name As String)
        Dim cat As New ADOX.Catalog

        ' if it already exits, then that's OK by me
        If File.Exists(mdb_name) Then
            Exit Sub
        End If

        cat.Create("Provider='Microsoft.Jet.OLEDB.4.0';Data Source='" & mdb_name & "'")
    End Sub

    '
    ' create a new table (if it doesn't already exist)
    '
    Public Sub CreateTable(ByVal mdb_name As String, ByVal table_name As String, ByVal DDL As String)
        If TableExists(mdb_name, table_name) = False Then
            Dim cmd As New OleDbCommand

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & mdb_name
                con.Open()
            End If

            ' build the new table 
            cmd.CommandText = "Create Table [" & table_name & "]" & DDL
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Delete a table if it exists
    '
    Public Sub DeleteTable(ByVal mdb_name As String, ByVal table_name As String)
        If TableExists(mdb_name, table_name) Then
            Dim cmd As New OleDbCommand

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & mdb_name
                con.Open()
            End If

            ' delete the table completely
            cmd.CommandText = "Drop Table [" & table_name & "]"
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Empty a table if it exists
    '
    Public Sub EmptyTable(ByVal mdb_name As String, ByVal table_name As String)
        If TableExists(mdb_name, table_name) Then
            Dim cmd As New OleDbCommand

            If con.State <> ConnectionState.Open Then
                con.ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & mdb_name
                con.Open()
            End If

            ' remove all rows from the table
            cmd.CommandText = "Delete from [" & table_name & "]"
            cmd.Connection = con
            cmd.ExecuteNonQuery()
        End If
    End Sub

    '
    ' Does the table already exist?
    '
    Public Function TableExists(ByVal mdb_name As String, ByVal table_name As String) As Boolean
        Dim dt As DataTable
        Dim dr As DataRow

        If con.State <> ConnectionState.Open Then
            con.ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & mdb_name
            con.Open()
        End If

        dt = con.GetOleDbSchemaTable(OleDbSchemaGuid.Tables, New Object() {Nothing, Nothing, Nothing, "TABLE"})
        For Each dr In dt.Rows
            If String.Compare(table_name, dr("TABLE_NAME").ToString, True) = 0 Then
                Return True
            End If
        Next

        Return False
    End Function

    '
    ' Open and return a DataTable
    '
    Function OpenTable(ByVal mdb_name As String, ByVal table_name As String) As DataTable
        Dim cmd As New OleDbCommand
        Dim dt As New DataTable

        If con.State <> ConnectionState.Open Then
            con.ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & mdb_name
            con.Open()
        End If

        cmd.Connection = con
        cmd.CommandText = "Select * from [" & table_name & "]"
        da.SelectCommand = cmd
        da.Fill(dt)

        dt.TableName = table_name
        dt.ExtendedProperties("CommandText") = cmd.CommandText
        Return dt
    End Function

    '
    ' Update the data in the underlying database
    '
    Sub UpdateTable(ByVal dt As DataTable)
        Dim cb As OleDbCommandBuilder

        da.SelectCommand.CommandText = dt.ExtendedProperties("CommandText").ToString
        cb = New OleDbCommandBuilder(da)
        cb.QuotePrefix = "["
        cb.QuoteSuffix = "]"
        da.Update(dt)
    End Sub
End Class

