' Add a Reference to System.DirectoryServices
Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports System.DirectoryServices
Imports System.Collections.Specialized

'
' A series of methods to get information about networked PCs.
' The supported techniques are:
'     Browsing the My Network Places
'     Querying an Active Directory Domain using LDAP
'     Querying a WinNT Domain using ADSI
'
Class GetNetInfo

#Region "API Region"
    'NET_API_STATUS NetServerEnum(
    '  LPCWSTR servername,
    '  DWORD level,
    '  LPBYTE* bufptr,
    '  DWORD prefmaxlen,
    '  LPDWORD entriesread,
    '  LPDWORD totalentries,
    '  DWORD servertype,
    '  LPCWSTR domain,
    '  LPDWORD resume_handle
    ');
    Declare Auto Function NetServerEnum Lib "Netapi32.dll" ( _
        ByVal servername As Integer, _
        ByVal level As Integer, _
        ByRef bufptr As IntPtr, _
        ByVal prefmaxlen As Integer, _
        ByRef entriesread As Integer, _
        ByRef totalentries As Integer, _
        ByVal servertype As Integer, _
        ByVal domain As String, _
        ByRef resume_handle As Integer _
    ) As Integer

    'NET_API_STATUS NetApiBufferFree(
    '  LPVOID Buffer
    ');
    Declare Function NetApiBufferFree Lib "Netapi32.dll" ( _
        ByVal Buffer As IntPtr _
    ) As Integer

    'typedef struct _SERVER_INFO_101 {
    '  DWORD sv101_platform_id;
    '  LPWSTR sv101_name;
    '  DWORD sv101_version_major;
    '  DWORD sv101_version_minor;
    '  DWORD sv101_type;
    '  LPWSTR sv101_comment;
    '} SERVER_INFO_101, *PSERVER_INFO_101, *LPSERVER_INFO_101;
    <StructLayout(LayoutKind.Sequential)> _
    Private Structure SERVER_INFO_101
        Dim sv101_platform_id As Integer
        Dim sv101_name As String
        Dim sv101_version_major As Integer
        Dim sv101_version_minor As Integer
        Dim sv101_type As Integer
        Dim sv101_comment As String
    End Structure

    Const SV_TYPE_ALL As Long = &HFFFFFFFF
    Const MAJOR_VERSION_MASK As Long = &HF
#End Region

    '
    ' Get info on the PCs from an NT-style domain using the LDAP WinNT provider
    '
    Public Sub GetPCDomInfo(ByVal dt As DataTable, ByVal DomainName As String)
        GetPCDomInfo(dt, DomainName, "")
    End Sub
    Public Sub GetPCDomInfo(ByVal dt As DataTable, ByVal DomainName As String, ByVal filter As String)
        Dim dirs, de As DirectoryEntry
        Dim dr As DataRow

        filter = filter.ToUpper
        Try
            dirs = New DirectoryEntry("WinNT://" & DomainName)
            For Each de In dirs.Children
                ' collect data on all of our Domain PCs
                If de.SchemaClassName = "Computer" Then
                    If filter = "" OrElse de.Name = filter Or de.Name Like filter Then
                        ' add it to our database
                        dr = dt.NewRow
                        dr("PCDomain") = DomainName
                        dr("PCName") = de.Properties("cn").Value.ToString
                        dr("OSVersion") = CDbl(de.Properties("operatingSystemVersion").Value.ToString.Split(" "c)(0))
                        dr("Services") = de.Properties("operatingSystem").Value.ToString & ", " & de.Properties("operatingSystemServicePack").Value.ToString
                        If de.Properties("description").Count > 0 Then
                            dr("PCComment") = de.Properties("description").Value.ToString
                        End If
                        dt.Rows.Add(dr)
                    End If
                End If
            Next
        Catch ex As Exception
            dr = dt.NewRow
            dr("PCDomain") = DomainName
            dr("PCComment") = Left("Error: " & ex.Message, 255)
            dt.Rows.Add(dr)
        End Try
    End Sub

    '
    ' Get the PC list from an Active Directory domain using an LDAP query
    '
    Public Sub GetPCOUInfo(ByVal dt As DataTable, ByVal DomainName As String, ByVal OU_Name As String)
        GetPCOUInfo(dt, DomainName, OU_Name, "", False)
    End Sub
    Public Sub GetPCOUInfo(ByVal dt As DataTable, ByVal DomainName As String, ByVal OU_Name As String, ByVal Filter As String)
        GetPCOUInfo(dt, DomainName, OU_Name, Filter, False)
    End Sub
    Public Sub GetPCOUInfo(ByVal dt As DataTable, ByVal DomainName As String, ByVal OU_Name As String, ByVal Filter As String, ByVal Svr_Only As Boolean)
        Dim entry, de As DirectoryEntry
        Dim srch As DirectorySearcher
        Dim result As SearchResult
        Dim ldap, buf, os As String
        Dim sc As New StringCollection
        Dim dr As DataRow

        Try
            ldap = "LDAP://" & DomainName & "/" & OU_Name

            ' set our starting place to search
            entry = New DirectoryEntry(ldap)
            srch = New DirectoryServices.DirectorySearcher(entry)
            srch.Filter = "(ObjectCategory=computer)"
            ' A work around for a bug... this shouldn't be needed
            srch.PageSize = 1000

            ' OK, let's do it
            Filter = Filter.ToUpper
            For Each result In srch.FindAll()
                de = result.GetDirectoryEntry

                buf = de.Properties("CN").Value.ToString
                If Filter = "" OrElse Filter = buf.ToUpper Or buf.ToUpper Like Filter Then
                    os = ""
                    If de.Properties("operatingSystem").Count > 0 Then
                        os = de.Properties("operatingSystem").Value.ToString
                    End If
                    If Not Svr_Only Or (Svr_Only And os Like "*Server*") Then
                        ' add it to our database
                        dr = dt.NewRow
                        dr("PCDomain") = DomainName
                        dr("PCName") = de.Properties("cn").Value.ToString
                        dr("OSVersion") = CDbl(de.Properties("operatingSystemVersion").Value.ToString.Split(" "c)(0))
                        dr("Services") = BuildServiceType(de.Properties("servicePrincipalName"))
                        If de.Properties("description").Count > 0 Then
                            dr("PCComment") = de.Properties("description").Value.ToString
                        End If
                        dt.Rows.Add(dr)
                    End If
                End If
            Next
        Catch ex As Exception
            dr = dt.NewRow
            dr("PCDomain") = DomainName
            ' NetList_ST only
            'dr("PCName") = New Random().Next.ToString
            dr("PCComment") = Left("Error: " & ex.Message, 255)
            dt.Rows.Add(dr)
        End Try
    End Sub

    '
    ' Get info on the PCs that you can see from the "My Network Places" browse list
    '
    Public Sub GetPCBrowseInfo(ByVal dt As DataTable, ByVal DomainName As String)
        GetPCBrowseInfo(dt, DomainName, "")
    End Sub
    Public Sub GetPCBrowseInfo(ByVal dt As DataTable, ByVal DomainName As String, ByVal filter As String)
        Dim level, MaxLenPref, ret As Integer
        Dim EntriesRead, TotalEntries, ResumeHandle, i As Integer
        Dim BufPtr, iPtr As IntPtr
        Dim dr As DataRow
        Dim TempNet As New SERVER_INFO_101

        ' let's do it!
        MaxLenPref = -1
        level = 101
        ret = NetServerEnum(0, level, BufPtr, MaxLenPref, EntriesRead, TotalEntries, SV_TYPE_ALL, DomainName, ResumeHandle)
        If ret = 0 Then
            ' record the results in our database
            For i = 0 To EntriesRead - 1

                ' some wizardry to "walk" the array of NETRESOURCE structures
                iPtr = New IntPtr(BufPtr.ToInt32 + (i * Marshal.SizeOf(TempNet)))
                TempNet = CType(Marshal.PtrToStructure(iPtr, GetType(SERVER_INFO_101)), SERVER_INFO_101)

                If filter = "" OrElse filter = TempNet.sv101_name.ToUpper Or TempNet.sv101_name.ToUpper Like filter Then
                    ' add it to our database
                    dr = dt.NewRow
                    dr("PCDomain") = DomainName
                    dr("PCName") = TempNet.sv101_name
                    dr("OSVersion") = CDbl(CStr(TempNet.sv101_version_major And MAJOR_VERSION_MASK) & "." & CStr(TempNet.sv101_version_minor))
                    dr("Services") = BuildType(TempNet.sv101_type)
                    If Not IsNothing(TempNet.sv101_comment) AndAlso TempNet.sv101_comment <> "" Then
                        dr("PCComment") = TempNet.sv101_comment
                    End If
                    dt.Rows.Add(dr)
                End If
            Next
            NetApiBufferFree(BufPtr)
        Else
            dr = dt.NewRow
            dr("PCDomain") = DomainName
            dr("PCComment") = Left("Error: " & New Win32Exception(ret).Message, 255)
            dt.Rows.Add(dr)
        End If
    End Sub

    Private Function BuildType(ByVal t As Integer) As String
        Dim Buf As String = ""
        Dim mask As Long
        Dim i As Integer

        If CBool(t And &H1000) Then
            Buf = "WinNT, "
        End If
        If CBool(t And &H400000) Then
            Buf = "Win9x, "
        End If
        If CBool(t And &H2000) And (Buf <> "Win9x, ") Then
            Buf = "WinFWG, "
        End If

        For i = 0 To 30
            mask = CLng(2 ^ i)
            If CBool(t And mask) Then
                Select Case i
                    Case 2
                        Buf = Buf & "SQL, "
                    Case 3
                        Buf = Buf & "PDC, "
                    Case 4
                        Buf = Buf & "BDC, "
                    Case 5
                        Buf = Buf & "Time, "
                    Case 6
                        Buf = Buf & "Apple, "
                    Case 7
                        Buf = Buf & "Novel, "
                    Case 8
                        Buf = Buf & "Member, "
                    Case 9
                        Buf = Buf & "Print, "
                    Case 10
                        Buf = Buf & "RAS, "
                    Case 11
                        Buf = Buf & "Xenix, "
                    Case 14
                        Buf = Buf & "Netware, "
                    Case 15
                        Buf = Buf & "Server, "
                    Case 18
                        Buf = Buf & "Browse, "
                End Select
            End If
        Next

        ' zap the trailing comma and space
        If Len(Buf) > 2 Then
            Buf = Left(Buf, Len(Buf) - 2)
        End If

        Return Buf
    End Function

    Private Function BuildServiceType(ByVal t As System.DirectoryServices.PropertyValueCollection) As String
        Dim temp As New ArrayList
        Dim obj As Object
        Dim service, ans As String

        For Each obj In t
            service = obj.ToString.Split("/"c)(0)
            If Not temp.Contains(service) Then
                temp.Add(service)
            End If
        Next

        ans = ""
        For Each service In temp
            ans &= service & ","
        Next

        Return ans.TrimEnd(","c)
    End Function
End Class
